import { knownFolders, path, Utils } from '@nativescript/core';
import { autoPlayProperty, loopProperty, LottieViewBase, progressProperty, srcProperty } from './lottie.common';
import { clamp } from './utils';
const appPath = knownFolders.currentApp().path;
export class LottieView extends LottieViewBase {
    get ios() {
        return this.nativeView;
    }
    createNativeView() {
        const view = CompatibleAnimationView.new();
        view.contentMode = 1;
        return view;
    }
    initNativeView() {
        super.initNativeView();
        this.nativeView.translatesAutoresizingMaskIntoConstraints = true;
    }
    [srcProperty.setNative](src) {
        if (!src) {
            this.nativeView.compatibleAnimation = null;
        }
        else if (src[0] === '{') {
            this.nativeView.compatibleAnimation = CompatibleAnimation.alloc().initWithJson(src);
        }
        else if (src.startsWith(Utils.RESOURCE_PREFIX)) {
            const resName = src.replace(Utils.RESOURCE_PREFIX, '');
            this.nativeView.compatibleAnimation = CompatibleAnimation.alloc().initWithNameBundle(resName.replace('.json', ''), NSBundle.mainBundle);
        }
        else if (src[0] === '~') {
            src = `${path.join(appPath, src.substring(2))}`;
            if (!/.(json|zip)$/.test(src)) {
                src += '.json';
            }
            try {
                this.nativeView.compatibleAnimation = CompatibleAnimation.alloc().initWithFilepath(src);
            }
            catch (error) {
                console.log(`Error initializing LottieView with src: ${src}`, error);
            }
        }
        else {
            try {
                this.nativeView.compatibleAnimation = CompatibleAnimation.alloc().initWithNameBundle(src, NSBundle.mainBundle);
            }
            catch (error) {
                console.log(`Error trying to set the iOS Lottie animation to LottieView with src: ${src}`, error);
            }
        }
    }
    [loopProperty.setNative](loop) {
        this.nativeView.loopAnimationCount = loop ? -1 : 0;
    }
    [autoPlayProperty.setNative](autoPlay) {
        if (autoPlay) {
            if (!this.isAnimating()) {
                this.playAnimation();
            }
        }
        else {
            if (this.isAnimating()) {
                this.cancelAnimation();
            }
        }
    }
    setColorValueDelegateForKeyPath(value, keyPath) {
        if (this.nativeView && value && keyPath && keyPath.length) {
            if (keyPath[keyPath.length - 1].toLowerCase() !== 'color') {
                keyPath.push('Color');
            }
            this.nativeView.setColorValueForKeypath(value.ios, CompatibleAnimationKeypath.alloc().initWithKeypath(keyPath.join('.')));
        }
    }
    setOpacityValueDelegateForKeyPath(value, keyPath) {
        if (this.nativeView && value && keyPath && keyPath.length) {
            if (keyPath[keyPath.length - 1].toLowerCase() !== 'opacity') {
                keyPath.push('Opacity');
            }
            this.nativeView.setFloatValueForKeypath(value, CompatibleAnimationKeypath.alloc().initWithKeypath(keyPath.join('.')));
        }
    }
    playAnimation() {
        setTimeout(() => {
            if (this.nativeView) {
                if (this.completionBlock) {
                    this.nativeView.playWithCompletion((animationFinished) => {
                        if (this.completionBlock) {
                            this.completionBlock(animationFinished);
                        }
                    });
                }
                else {
                    this.nativeView.play();
                }
            }
        }, 0);
    }
    playAnimationFromProgressToProgress(startProgress, endProgress) {
        if (this.nativeView) {
            startProgress = startProgress ? clamp(startProgress) : 0;
            endProgress = endProgress ? clamp(endProgress) : 1;
            if (this.completionBlock) {
                this.nativeView.playFromProgressToProgressCompletion(startProgress, endProgress, this.completionBlock);
            }
            else {
                this.nativeView.playFromProgressToProgressCompletion(startProgress, endProgress, null);
            }
        }
    }
    cancelAnimation() {
        if (this.nativeView) {
            this.nativeView.pause();
        }
    }
    isAnimating() {
        return this.nativeView ? this.nativeView.isAnimationPlaying : false;
    }
    [progressProperty.setNative](value) {
        this.nativeView.currentProgress = value;
    }
    set speed(value) {
        if (this.nativeView && value) {
            this.nativeView.animationSpeed = value;
        }
    }
    get speed() {
        return this.nativeView ? this.nativeView.animationSpeed : undefined;
    }
    get duration() {
        return this.nativeView && this.nativeView.animationDuration;
    }
    set contentMode(mode) {
        if (this.nativeView) {
            this.nativeView.contentMode = mode;
        }
    }
}
//# sourceMappingURL=lottie.ios.js.map